# Forms the starctural design from Box, Hunter & Hunter given
# the number of factors (k) and the fraction (f). 
# Libraries
#############
library(MASS)
library(Matrix) 
library(mvtnorm)
library(safeBinaryRegression)
# library(matrixStats)
library(Hmisc)

# Returns x - the all-plus-sign defining relation half block of runs=2^(k-f)
relations.def <-function(k,f){
  runs<-2^(k-f)
  x<-matrix(ncol=k+1,nrow=runs)
  x[,1]<-rep(1,runs)
  for(i in 1:(k-f)){
    xx<-c(rep(1,2^(k-f-i)),rep(-1,2^(k-f-i)))
    x[,i+1]<-rep(xx,2^(i-1))
  }
  x<-x[,-1]
  #runs=8
  if(k==5 & f==2){
    x[,4]<-x[,1]*x[,2]
    x[,5]<-x[,1]*x[,3]
  }
  if(k==6 & f==3){
    x[,4]<-x[,1]*x[,2]
    x[,5]<-x[,1]      *x[,3]
    x[,6]<-      x[,2]*x[,3]
  }
  if(k==7 & f==4){
    x[,4]<-x[,1]*x[,2]
    x[,5]<-x[,1]      *x[,3]
    x[,6]<-      x[,2]*x[,3]
    x[,7]<-x[,1]*x[,2]*x[,3]
  }
  
  #runs=16
  if(k==11 & f==7){
    x[,5]<-x[,1]*x[,2]*x[,3]
    x[,6]<-      x[,2]*x[,3]*x[,4]
    x[,7]<-x[,1]      *x[,3]*x[,4]
    x[,8]<-x[,1]*x[,2]      *x[,4]
    x[,9]<-x[,1]*x[,2]*x[,3]*x[,4]
    x[,10]<-x[,1]*x[,2]
    x[,11]<-x[,1]*x[,3]
  }
  if(k==10 & f==6){
    x[,5]<-x[,1]*x[,2]*x[,3]
    x[,6]<-      x[,2]*x[,3]*x[,4]
    x[,7]<-x[,1]      *x[,3]*x[,4]
    x[,8]<-x[,1]*x[,2]      *x[,4]
    x[,9]<-x[,1]*x[,2]*x[,3]*x[,4]
    x[,10]<-x[,1]*x[,2]  
  }
  if(k==9 & f==5){
    x[,5]<-x[,1]*x[,2]*x[,3]
    x[,6]<-      x[,2]*x[,3]*x[,4]
    x[,7]<-x[,1]      *x[,3]*x[,4]
    x[,8]<-x[,1]*x[,2]      *x[,4]
    x[,9]<-x[,1]*x[,2]*x[,3]*x[,4]
  }
  if(k==8 & f==4){
    x[,5]<-      x[,2]*x[,3]*x[,4]
    x[,6]<-x[,1]      *x[,3]*x[,4]
    x[,7]<-x[,1]*x[,2]*x[,3]
    x[,8]<-x[,1]*x[,2]      *x[,4]
  }
  if(k==7 & f==3){
    x[,5]<-x[,1]*x[,2]*x[,3]
    x[,6]<-      x[,2]*x[,3]*x[,4]
    x[,7]<-x[,1]      *x[,3]*x[,4]
    }
  if(k==6 & f==2){
    x[,5]<-x[,1]*x[,2]*x[,3]
    x[,6]<-      x[,2]*x[,3]*x[,4]
  }
  if(k==5 & f==1){
    x[,5]<-x[,1]*x[,2]*x[,3]*x[,4]
    }
  #runs=32
  if(k==11 & f==6){
    x[,6]<-x[,1]*x[,2]*x[,3]
    x[,7]<-      x[,2]*x[,3]*x[,4]
    x[,8]<-            x[,3]*x[,4]*x[,5]
    x[,9]<-x[,1]      *x[,3]*x[,4]
    x[,10]<-x[,1]           *x[,4]*x[,5]
    x[,11]<-     x[,2]      *x[,4]*x[,5]
  }
  if(k==10 & f==5){
    x[,6]<-x[,1]*x[,2]*x[,3]*x[,4]
    x[,7]<-x[,1]*x[,2]*x[,3]      *x[,5]
    x[,8]<-x[,1]*x[,2]      *x[,4]*x[,5]
    x[,9]<-x[,1]      *x[,3]*x[,4]*x[,5]
    x[,10]<-     x[,2]*x[,3]*x[,4]*x[,5]
  }
  if(k==9 & f==4){
    x[,9]<-x[,1]*x[,2]*x[,3]      *x[,5]
    x[,8]<-x[,1]*x[,2]      *x[,4]*x[,5]
    x[,7]<-x[,1]      *x[,3]*x[,4]*x[,5]
    x[,6]<-     x[,2]*x[,3]*x[,4]*x[,5]
  }
  if(k==8 & f==3){
    x[,6]<-x[,1]*x[,2]*x[,3]
    x[,7]<-x[,1]*x[,2]      *x[,4]
    x[,8]<-     x[,2]*x[,3]*x[,4]*x[,5]
  }
  if(k==7 & f==2){
    x[,6]<-x[,1]*x[,2]*x[,3]*x[,4]
    x[,7]<-x[,1]*x[,2]      *x[,4]*x[,5]
  }
  if(k==6 & f==1){
    x[,6]<-x[,1]*x[,2]*x[,3]*x[,4]*x[,5]
  }
  return(x)
}

# Checks if the basic blocks are already foldover pairs 
is.foldover<-function(M){
  out<-ifelse(find.first.row(-M[1,],M)$found==FALSE,FALSE,TRUE)
return(out)
}


# Finds duplications among the blocks. Returns a half of
# the blocks which are unique.
########################################################
find.dual.block<-function(blocks,full.design){
  dual.blocks<-c()
  runs.in.full<-matrix(ncol=2*nblocks,nrow=2*runs)
  for (h1 in 1:(2*nblocks)){
    runs.in.full[,h1]<-apply(blocks[h1,,],1,find.first.row,full.design)} # problematic, find.first row returns a LIST
  for (h1 in 1:(2*nblocks-1)){
    if(all(h1!=dual.blocks)){
      h2<-h1+1
      while(!identical(sort(runs.in.full[,h1]),sort(runs.in.full[,h2]))){h2=h2+1}
      dual.blocks<-rbind(dual.blocks,c(h1,h2))
    }
  }
  return(dual.blocks)
}


# This function forms the factors according to the defining relations
# given a vector of signs (one of 2^f).
# returns the block (nrow=2*runs,ncol=k) - foldover pair

generator<-function(sign,ck,cf,cruns,base.design,x){
  block.mat<-matrix(nrow=2*cruns,ncol=ck)
  block.mat[(1:cruns),(1:dim(base.design)[2])]<-base.design
  for (c in (ck-cf+1):ck){
    block.mat[1:cruns,c] <-sign[c-(ck-cf)]*x[,c]
  }
  
  block.mat[(cruns+1):(2*cruns),]<- - block.mat[1:cruns,]
  
  return (block.mat)
}

generator.without.foldover<-function(sign,ck,cf,cruns,base.design,x){
  block.mat<-matrix(nrow=cruns,ncol=ck)
  block.mat[(1:cruns),(1:dim(base.design)[2])]<-base.design
  for (c in (ck-cf+1):ck){
    block.mat[1:cruns,c] <-sign[c-(ck-cf)]*x[,c]
  }
  
  return (block.mat)
}
# Calculation of Hamming distance between two vectors

HD<-function(x,y){
  return(sum(x!=y))
}

# Calculation of Hamming distances dist'n between two blocks.
# returns the vector of distances.

HD.blocks = function(B1,B2){
  d = apply(B2,1,HD,B1[1,])
  return(d)
}

# Choosing the next block to run, given an initial block B1.
# returns the chosen block index.

choose.blocks<-function(list1){
  B1<-list1$B1
  used<-list1$used.blocks
  runs<-list1$runs
  nblocks<-list1$nblocks
  blocks<-list1$blocks
  dist2.mat<-matrix(ncol=2*runs,nrow=nblocks)
  dist2.mat.sorted<-matrix(ncol=2*runs,nrow=nblocks)
  for(i in 1:nblocks){
    if(!used[i]){
      dist2.mat[i,]<-HD.blocks(B1,blocks[i,,])
      dist2.mat.sorted[i,]<-sort(HD.blocks(B1,blocks[i,,]))}
  }
  u<-unique(dist2.mat.sorted,MARGIN=1)
  order.u<-u[do.call(order, lapply(1:NCOL(u), function(i) u[, i])), ]
  order.u<-na.omit(order.u)
  max.dist<-order.u[nrow(order.u),]  
  pool<-c() 
  for(i in 1:nblocks){
    if(isTRUE(all.equal(dist2.mat.sorted[i,],max.dist)))
      pool<-cbind(pool,i)}
  pool<-as.vector(pool)   
  ifelse(length(pool)==1,next.block.ind<-pool,next.block.ind<-sample(pool,1))
  return(next.block.ind)
}



# Forms a 2^(k-f) design, rep replications. 
# Returns the formed design and vector of chosen blocks
# valid "type" arguments are: 
# "FFD" for random selection of blocks
# "HDFFD" for smart selection of blocks
# "RAN" for random selection of runs
form.design<-function(list){
  type <- list$type
  nblocks<-list$nblocks
  blocks<-list$blocks
  cblocks<-list$blocks
  rep<-list$rep
  k<-list$k
  f<-list$f
  runs<-list$runs
    
  
  if(f>0){
    if(nblocks==1){
      temp.design<-blocks[1,,]
      for(r in 1:(rep-1)){
        temp.design<-rbind(temp.design,blocks[1,,])
      }
      chosen.blocks<-rep(1,rep)
    }
    if(nblocks>1){
      temp.design<-matrix(ncol=k,nrow=rep*2^k)
      if(isTRUE(type=='HDFFD') | is.null(type)){
        chosen.blocks<-c()
        B1.ind<-rep(NA,rep)
        
        # For each replication, choose randomly the first block,
        # set to "FALSE" the used.blocks factor,
        # and choose the next blocks for the replication
        for(r in 1:rep){
          B1.ind[r]<-sample(c(1:nblocks),1)
          B1<-blocks[B1.ind[r],,]
          used.blocks<-rep(FALSE,nblocks)
          used.blocks[B1.ind[r]]<-TRUE
          chosen.blocks[1+(r-1)*nblocks]<-B1.ind[r]
          temp.design[(1+(r-1)*2^k):((r-1)*2^k+2*runs),]<-B1
          
          list1<- c(list,list(temp.design=temp.design,used.blocks=used.blocks,B1=B1))
          for(i in 2:nblocks){
            list1$B1<-temp.design[((r-1)*2^k+1):((r-1)*2^k+(i-1)*2*runs),]
            list1$used.blocks <- used.blocks
            next.block.ind<-choose.blocks(list1)
            
            temp.design[(r-1)*2^k+(((i-1)*2*runs+1):(i*2*runs)),]<-blocks[next.block.ind,,]
            used.blocks[next.block.ind]<-TRUE
            chosen.blocks[(r-1)*nblocks+i]<-next.block.ind
          }
        }
      }
      if(isTRUE(type=='FFD')){
        chosen.blocks <- c()
        for(r in 1:rep){
          chosen.blocks <- c(chosen.blocks,sample(c(1:nblocks)))
          for(i in 1:nblocks){
            next.block.ind <- chosen.blocks[i]
            temp.design[(r-1)*2^k+(((i-1)*2*runs+1):(i*2*runs)),]<-blocks[next.block.ind,,]
          }
        }
      }
      if(isTRUE(type=='RAN')){
        chosen.blocks <- c()
        for(r in 1:rep){
          runs.random.order<-sample(c(1:2^k))
          temp.design[((r-1)*2^k+1):(r*2^k),]<-full.design0[runs.random.order,-1]
        }
      }
    }
  }
  if(f<=0){
    temp.design<-c()
    if(dim(cblocks)[3]!=1){
      chosen.blocks<-c()
      for (r in 1:rep){temp.design<-rbind(temp.design,blocks[1,,])}
      }
    if(dim(cblocks)[3]==1){
      chosen.blocks<-c()
      temp.design<-rep(blocks[1,,],rep)
      }
  }
  return(list(design=temp.design,chosen.blocks=chosen.blocks))
}

form.random.design <- function(list){
  # nblocks<-l$nblocks
  # blocks<-l$blocks
  # runs<-l$runs
  
  rep<-list$rep
  k<-list$k1
  f<-list$f1
  full.design0<-list$full.design0
  temp.design0<-matrix(ncol=k+1,nrow=rep*2^k)
  # runs.random.order<-matrix(nrow=2^k,ncol=rep)
  # runs.indices<-c()
  for(r in 1:rep){
    # runs.random.order[,r]<-sample(c(1:2^k))
    # temp.design0[((r-1)*2^k+1):(r*2^k),]<-full.design0[runs.random.order[,r],]
    # runs.indices<-cbind(runs.indices,runs.random.order[,r])
    runs.random.order<-sample(c(1:2^k))
    temp.design0[((r-1)*2^k+1):(r*2^k),]<-full.design0[runs.random.order,]
  }
  # return(list(design=temp.design,runs.indices=runs.indices))
  return(temp.design0)
  
}

#check if factor j creates SEPARATION in the data until block bl
check.sep<-function(lst){
  ind<-(1:(lst$bl*2*lst$runs))
  tab<-table(lst$y[ind],lst$design[ind,lst$j])
  out<-ifelse(all(lst$y[ind]==0),TRUE,xor(tab[2,1]==0,tab[2,2]==0))  
return(out)
}

# # checks if it is possible to fit a regression model to data
# reg.fit.possible<-function(base.design,full.design,y){
#   ncomb<-nrow(base.design)
#   comb.mat<-matrix(NA,ncol=ncomb,nrow=nrow(full.design))
#   sum0<-c()
#   for (i in 1:ncomb){
#     ind<-find.all.rows(base.design[i,],full.design)$ind
#     comb.mat[1:length(ind),i]<-ind
#     sum0[i]<-sum(y[as.vector(na.omit(comb.mat[,i]))])
#   }
#  out<-list(pos=!any(sum0==0),sum0=sum0)
#  return(out) 
# }

reg.fit.possible <- function(data,fmula) {
  out <- tryCatch(
    {
      # Just to highlight: if you want to use more than one 
      # R expression in the "try" part then you'll have to 
      # use curly brackets.
      # 'tryCatch()' will return the last evaluated expression 
      # in case the "try" part was completed successfully
      
      # message("This is the 'try' part")
      list(error=FALSE,warnings=FALSE,g=glm(fmula,data=data,family=binomial(link="probit"),separation = "test"))       

      # that will be returned in case there is no condition 
      # (e.g. warning or error). 
      # You don't need to state the return value via `return()` as code 
      # in the "try" part is not wrapped insided a function (unlike that
      # for the condition handlers for warnings and error below)
    },
    error=function(cond) {
      # message(paste("URL does not seem to exist:", url))
      # message("Here's the original error message:")
      # message(cond)
      # Choose a return value in case of error
      ER<-TRUE
      return(list(error=ER,warnings=FALSE,g=NA))
    },
    warning=function(cond) {
      # message(paste("URL caused a warning:", url))
      # message("Here's the original warning message:")
      # message(cond)
      # Choose a return value in case of warning
      # WAR<-TRUE
      WAR<-TRUE
      return(list(error=FALSE,warnings=WAR,g=NA))
    },
    finally={
      # NOTE:
      # Here goes everything that should be executed at the end,
      # regardless of success or error.
      # If you want more than one expression to be executed, then you 
      # need to wrap them in curly brackets ({...}); otherwise you could
      # just have written 'finally=<expression>' 
      # message(paste("Processed URL:", url))
      # message("Some other message at the end")
      # out<-list(g=g,ERROR=ER,WARNING=WAR)
    }
  )    
  return(out)
}
#
first.strong<-function(v){
  ind<-ifelse(all(!v),length(v)+1,min(which(v)))
  return(ind)
}
#
always.strong<-function(v){
  bl<-length(v)
  while(all(v[bl:length(v)])) bl<-bl-1
  return (bl+1)
}


# Finds the row equal to v in a matrix x.
# returns the index of the row in x.

find.first.row<-function(v,x){
  found<-FALSE
  if(is.vector(x)){
    if(all(x!=v)){out<-NA}
    if(any(x==v)){out<-min(which(x==v));found<-TRUE}
  }  
  if(is.matrix(x)){
    for(i in 1:nrow(x)){ 
      if(isTRUE(all.equal(v,x[i,]))){out<-i;found<-TRUE;break}
    }
    if(!found)out<-NA
  }
  return (list(ind=out,found=found)) 
}

# Find the rows in x which are equal to vector v

find.all.rows<-function(v,x){
  v<-as.vector(v)
  found<-FALSE
  if(is.vector(x)){
    if(all(x!=v)){out<-NA}
    if(any(x==v)){out<-which(x==v);found<-TRUE}
  }  
  if(is.matrix(x)){
    out<-c()
    for(i in 1:nrow(x)){ 
      if(isTRUE(all.equal(v,as.vector(x[i,])))){out<-c(out,i);found<-TRUE}
    }
    if(!found)out<-NA
  }
  return (list(ind=out,found=found)) 
}

# Finds the number of rows equals to v in x
# Application: find number of plays with a specific
# combination in a design matrix with outcomes vector y.

count.comb.suc<-function(v,x,y){
  ind<-find.all.rows(v,x)$ind
  out1<-length(ind)
  out2<-sum(y[ind])
  return(list(n.comb=out1,n.suc=out2))
}


# proportion of +1 out of no. success runs for every factor 
prop1<-function(v){
  u<-sum(v[v==1])
  return(u/length(v))
}

# the same but with -1
prop.minus1<-function(v){
u<-sum(v[v==-1])
return(-u/length(v))
}


# David's function:
# The following function converts a prior distribution with equally
# weighted values  t  into a posterior distribution on those values
# where  wt  gives the weight for each point in  t  from the likelihood.
# The function returns the ordered values of  t  and the cumulative
# posterior probability of being less than or equal to  t.

post.t <- function(t,wt) {
  
  t1 <- sort(t)
  wt.ordered <- wt[order(t)]
  post.prob <- wt.ordered[1]
  for (j in 2:length(t)) {
    post.prob <- c(post.prob,post.prob[j-1]+wt.ordered[j])
  }
  return(cbind(t1,post.prob))		
}

# This function sets a "good" sample from the posterior.
# gets initial values: tstart,mu.resample,cov.resample
# and returns the sample, and the covariance of the resample.

resample<- function(list){
  tstart<-list$tstart
  mu.resample<-list$mu.resample
  cov.resample<-list$cov.resample
  Flag<-list$Flag
  WARNING<-list$WARNING
  design0<-list$design0
  w.res<-list$w.res
  a<-list$a
  m<-list$m
  y<-list$y
  LIK.0<-list$LIK.0
  count<-list$count
  runs<-list$runs
  
  while((sum(w.res^2)>=cutoff || max(w.res)>max.w || Flag) & count < resample.count){
    sigma<-a*cov.resample
    mu.corrected<- m + mu.resample
    b.prior.res<-mvrnorm(n, mu=mu.corrected, Sigma=sigma)
    prior.density<-rep(NA,n)
    resampling.density<-rep(NA,n)
  
    for(i in 1:n){
      prior.density[i]<-dmvnorm(b.prior.res[i,],mean=mu,sigma=cov_mtrx)
      resampling.density[i]<-dmvnorm(b.prior.res[i,],mean=mu.corrected, sigma=sigma)
    }
    if(!LIK.0){
    log.lik<-rep(NA,n)
    for(i in 1:n){
      log.lik[i]<-sum(y[1:((tstart-1)*2*runs)]*log(pnorm(design0[1:((tstart-1)*2*runs),]%*%b.prior.res[i,]))
                           +(1-y[1:((tstart-1)*2*runs)])*log(1-pnorm(design0[1:((tstart-1)*2*runs),]%*%b.prior.res[i,])))  
    }
    lik<-exp(log.lik)
    lik[is.na(lik)]<-0  
    unw<-lik*(prior.density/resampling.density)
    if(sum(unw)!=0) w.res<-unw/sum(unw)
    if(sum(unw)==0) LIK.0<-TRUE
    }
    if(LIK.0) {
      N<-20
      ngroups<-ceiling((tstart-1)/N)
#       ngroups<-10
#       N<-ceiling((tstart-1)/ngroups)
      frac<-c(c(1:(ngroups-1))*N,tstart-1)
      w.temp.mat<-matrix(nrow=n,ncol=2)
      lik<-rep(NA,n)
      log.lik<-rep(NA,n)
      j<-frac[1]
      for (i in 1:n){
         log.lik[i]<-sum(y[1:((j-1)*2*runs)]*log(pnorm(design0[1:((j-1)*2*runs),]%*%b.prior.res[i,]))
                              +(1-y[1:((j-1)*2*runs)])*log(1-pnorm(design0[1:((j-1)*2*runs),]%*%b.prior.res[i,])))  
      }
      lik<-exp(log.lik)*(prior.density/resampling.density)
      lik[is.na(lik)]<-0
      w.temp.mat[,1]<-lik/sum(lik)
      for(s in 2:ngroups){
        frac.ind<-c((frac[s-1]*2*runs+1):(frac[s]*2*runs))
        for (i in 1:n){     
          log.lik[i]<-sum(y[frac.ind]*log(pnorm(design0[frac.ind,]%*%b.prior.res[i,]))
                              +(1-y[frac.ind])*log(1-pnorm(design0[frac.ind,]%*%b.prior.res[i,])))  
        }
        lik<-exp(log.lik)
        lik[is.na(lik)]<-0
        w.temp.mat[,2]<-w.temp.mat[,1]*lik/sum(w.temp.mat[,1]*lik)
        w.temp.mat[,1]<-w.temp.mat[,2]
      }
        
      w.res<-w.temp.mat[,2]
    }

   
    con1<-con2<-con3<-rep(FALSE,K+1)
    for(d in 1:(K+1)){
      b.w<-cbind(sort(b.prior.res[,d]),w.res[order(b.prior.res[,d])])
      cdf<-post.t(b.prior.res[,d],w.res)
      #plot(cdf[,2]~cdf[,1],xlab="sorted sample",ylab="cumulative weights",cex.axis=0.8,cex.lab=0.8,col=c(rep("blue",tail),rep("black",n-2*tail),rep("red",tail)),main=bquote(~beta[.(d-1)]))
      con1[d]<-max(sum(b.w[1:tail,2]),sum(b.w[(n-tail+1):n,2]))>max.w.tail
      con3[d]<-max(sum(b.w[1:tail,2]),sum(b.w[(n-tail+1):n,2]))>max.tail.shift
      if(con1[d]) a[d]<-max(a[d]+c.inc,0.3)
      direc<-ifelse(sum(b.w[1:tail,2])<sum(b.w[(n-tail+1):n,2]),1,-1)
      if(con3[d]) m[d]<-m[d]+direc*0.5*sqrt(sigma[d,d])
      }
      if(any(con1)||any(con3)) {
        Flag=TRUE
        for(d in 1:(K+1)){
        con2[d]<-min(sum(b.w[1:tail1,2]),sum(b.w[(n-tail1+1):n,2]))<min.w.tail
        if(con2[d]) a[d]<-max(a[d]-c.dec,0.3)
        }
      }
    if(all(!con1)&all(!con3)) Flag=FALSE
    count<-count+1
  }
  rstop<-"sample meets criteria"
  #if(WARNING){rtsop<-"sum of weights is zero"}
  if(count==resample.count){it=resample.count;rstop<-"reached max number of iterations"}
 return(list(b.prior.res=b.prior.res,sigma=sigma,rstop=rstop,a=a,
             it=count,m=m, prior.density=prior.density,
             w.res=w.res,LIK.0=LIK.0,
             resampling.density=resampling.density,log.lik=log.lik))
}
 
# Decides which levels combination to play according to Thompson Sampling.
# Basis for the same function only with interactions
# ****CURRENTLY NOT AT USE****

Thompson.with.interactions<-function(bl.start,k,f,k1,f1,b.post,sigma.post,
                                     number.of.strong,design,ind.strong){
  
  post.sample<-rmvnorm(2*cruns,mean=post.mean.temp[bl,],sigma=diag(post.sd.temp[bl,]))
  levels<-matrix(nrow=2^number.of.strong,ncol=number.of.strong)
  for(j in 1:number.of.strong){
    xx<-c(rep(1,2^(number.of.strong-j)),rep(-1,2^(number.of.strong-j)))
    levels[,j]<-rep(xx,2^(j-1))
  }
  for(i in 1:(2*runs)){
    temp<-matrix(ncol=k,nrow=(2^number.of.strong))
    for(j in 1:(2^number.of.strong)){
      temp[j,-ind.strong]<-design[bl.start+i,-ind.strong]
      temp[j,ind.strong]<-levels[j,]
    }
    temp<-cbind(rep(1,2^number.of.strong),temp)
    pt<-pnorm(temp%*%post.sample[i,])                   
    comb.max<-temp[which.max(pt),]                                
    y[bl.start+i,]<-rbinom(1,1,comb.max%*%b)                    
    design0[bl.start+i,]<-temp[which.max(pt),]
  }
  design<-design0[,-1]
  
  return(list(design=design,design0=design0,data=data,post.sample=post.sample))
  
}

# CI for x = number of successes in a binomial experiment
# with n trials each.  based on Normal approximation to the binomial probability.

binom.CI<-function(x,n,level){
  m<-length(x)
  a<-1-level
  p.hat<-x/n
  p.mle<-mean(p.hat)
  d<-qnorm(1-a/2)*sqrt(p.mle*(1-p.mle)/(m*n))
  return(list(CI=c(p.mle-d,p.mle+d),MLE=p.mle,Std.error=sqrt(p.mle*(1-p.mle)/(m*n))))
}

posterior<-function(list2,samp=TRUE){
  bl<-list2$bl 
  cruns<-list2$cruns
  SEPARATION<-list2$SEPARATION
  y<-list2$y
  design0<-list2$design0
  design<-design0[,-1]
  cBA<-list2$cBA
  
  data.to.block.ind<-c(1:(bl*2*cruns))
##################################################
# There is a problem with the Normal approximation.
##################################################
  # if(!SEPARATION){
  #   # prior:
  #   b0<-mu 
  #   v0<-cov_mtrx
  #   # likelihood:
  #   g<-glm(y[data.to.block.ind]~design[data.to.block.ind,],family=binomial(link="probit"))  
  #   b.hat<-as.vector(coef(summary(g))[,1])
  #   LP<-design0[data.to.block.ind,]%*%b.hat
  #   p.hat<-pnorm(LP)
  #   w.vec<-rep(NA,bl*2*cruns)
  #   for(j in 1:(bl*2*cruns)){
  #     w.vec[j]<-(dnorm(design0[j,]%*%b.hat))^2/(pnorm(design0[j,]%*%b.hat)*(1-pnorm(design0[j,]%*%b.hat)))
  #   }
  #   W.mat<-as.matrix(Diagonal(x=w.vec))
  #   v<-solve((t(design0[data.to.block.ind,])%*%W.mat)%*%design0[data.to.block.ind,])
  #   
  #   # posterior
  #   b.post<-(solve(v0+v)%*%v0)%*%b.hat + (solve(v0+v)%*%v)%*%b0
  #   sigma.post<- solve(solve(v)+solve(v0))
  #   update.mean<-b.post
  #   update.median<-b.post
  #   update.sd<-diag(sigma.post)
  #   update.post.prob<-1-pnorm(0,update.mean,update.sd)
  #   update.est<-"Normal"
  #   BA<-"Normal approximation"
  #    
  #   # If required, draw a sample from the posterior
  #   if(samp) post.sample<-rmvnorm(2*cruns,b.post,sigma.post)
  # }
  # 
  # if(SEPARATION){
  ###############################################################
    if(bl>1){
        BA1<-c(cBA,list(nblocks=bl,y=y[data.to.block.ind],runs=cruns,
                      design0=design0[data.to.block.ind,],RESAMPLE=RESAMPLE))
        BA<-Bayesian.Analysis.update(BA1)
        update.mean<-BA$post.mean[bl+1,]
        update.median<-BA$post.median[bl+1,]
        update.sd<-BA$post.sd[bl+1,]
        update.mean.p<-BA$p.post.mean[bl+1,]
        update.sd.p<-BA$p.post.sd[bl+1,]
        update.est<-"Discrete"
        update.post.prob<-BA$post.prob[bl+1,]
        
    }
    if(bl==1){
      ls<-list(nblocks=bl,y=y[data.to.block.ind],runs=cruns,
               design0=design0[data.to.block.ind,])
      BA<-Bayesian.Analysis.first(ls)
      update.mean<-BA$post.mean
      update.median<-BA$post.median
      update.sd<-BA$post.sd
      update.mean.p<-BA$p.post.mean
      update.sd.p<-BA$p.post.sd
      update.est<-"Discrete"
      update.post.prob<-BA$post.prob

    }
    if(samp){
    cdf.b<-cumsum(BA$w[,ncol(BA$w)])
    ru<-runif(2*cruns,0,1)
    ind<-c()
    for(m in 1:(2*cruns))ind[m]<-min(which(cdf.b>ru[m]))
    
    if(BA$RESAMPLE)post.sample<-BA$b.prior.res[ind,]
    if(!BA$RESAMPLE)post.sample<-BA$b.prior[ind,]
    }
#####################################################
# }
#####################################################
  if(!samp)post.sample<-"not requested"
  
  
  return(list(update.mean=update.mean,update.median=update.median,update.sd=update.sd,
              update.est=update.est,post.sample=post.sample,BA=BA,
              update.post.prob=update.post.prob,
              update.mean.p=update.mean.p,update.sd.p=update.sd.p))
}

# Given the levels of nmain main effects in a design matrix,
# returns interactions levels and the interaction design matrix

form.interactions<-function(design,nmain){
  int.mat<-matrix(nrow=nmain,ncol=2^nmain-1)
  if(nmain==2){
    design[,3]<-design[,1]*design[,2]
    int.mat[,3]<-c(1,2)
  }
  if(nmain==3){
    design[,4]<-design[,1]*design[,2]
    design[,5]<-design[,1]*          design[,3]
    design[,6]<-           design[,2]*design[,3]
    design[,7]<-design[,1]*design[,2]*design[,3]
    
    int.mat[,4]<-c(1,2,NA)
    int.mat[,5]<-c(1,3,NA)
    int.mat[,6]<-c(2,3,NA)
    int.mat[,7]<-c(1,2,3)
  }
  
  return(list(design=design,int.mat=int.mat))
}

# for a fiven M (a record of the boolean Thompson.strong matrix)
# returns a length k vector with the first time (block number) a factor began
# to be allocated according to Thompson.
compute.Thompson.first<- function(M){
  S<-apply(M,2,sum)
  S[S!=0]<-apply(M,2,which.max)[S!=0]
  S[S==0]<--1
  return (S)
}

# for a given b.sample matrix, compute the 2^k probabilities
# induced by each sample point (row of b.sample), for every 
# combination of the design matrix.
# Returns a (n x 2^k) matrix.
induced.ps<-function(b.sample){
  full.design0<-matrix(ncol=K+1,nrow=2^K) 
  full.design0[,1]<-rep(1,2^K)
  for(i in 1:K){
    xx<-c(rep(1,2^(K-i)),rep(-1,2^(K-i)))
    full.design0[,i+1]<-rep(xx,2^(i-1))
  }
  ps.mat<-pnorm(t(full.design0%*%t(b.sample)))
  return(ps.mat)
}

# Computes the q quantile of a distribution with 
# discrete representation t,wt.

postQ <- function(t,wt,q){
  cdf <- post.t(t,wt)
  return(as.numeric(cdf[min(which(cdf[,2] > q)),1]))
}

# computes the first time the posterior weight is above 95%.

high.post.w <- function (v,direc = NULL){
  hw <- 0.975
  lw <- 1-hw
  if (is.null(direc)){
    con <- any(v > hw) | any(v < lw)
    if(con) time <- which.max(v > hw | v < lw)
    if(!con) time <- max.blocks + 1
    return (time)
    break
  }
  if (direc =="pos"){
    con <- any(v > hw)
    if(con) time <- which.max(v > hw)
    if(!con) time <- max.blocks + 1
    return (time)
    break
  }
  if (direc =="neg"){
    con <- any(v < lw)
    if(con) time <- which.max(v < lw)
    if(!con) time <- max.blocks + 1
    return (time)
    break
  }
}